from enum import StrEnum


class PropertyType(StrEnum):
    TITLE = "title"
    RICH_TEXT = "rich_text"
    SELECT = "select"
    MULTI_SELECT = "multi_select"
    STATUS = "status"
    NUMBER = "number"
    DATE = "date"
    CHECKBOX = "checkbox"
    URL = "url"
    EMAIL = "email"
    PHONE_NUMBER = "phone_number"
    PEOPLE = "people"
    CREATED_BY = "created_by"
    LAST_EDITED_BY = "last_edited_by"
    CREATED_TIME = "created_time"
    LAST_EDITED_TIME = "last_edited_time"
    LAST_VISITED_TIME = "last_visited_time"
    FORMULA = "formula"
    ROLLUP = "rollup"
    FILES = "files"
    RELATION = "relation"
    BUTTON = "button"
    LOCATION = "location"
    PLACE = "place"
    VERIFICATION = "verification"
    UNIQUE_ID = "unique_id"
